
#include <AIDA/IAnalysisFactory.h>
#include <AIDA/ITreeFactory.h>
#include <AIDA/ITree.h>
#include <AIDA/IHistogramFactory.h>
#include <AIDA/IHistogram1D.h>
#include <AIDA/IHistogram2D.h>
#include <AIDA/IProfile1D.h>
#include <AIDA/ITupleFactory.h>
#include <AIDA/ITuple.h>
#include <AIDA/IPlotterFactory.h>
#include <AIDA/IPlotter.h>
#include <AIDA/IPlotterRegion.h>

#include <Lib/Arguments.h>
#include <Lib/Randoms.h>
#include <Lib/Printer.h>
#include <Lib/Debug.h>
#include <Lib/smanip.h>

#include <iostream>

//////////////////////////////////////////////////////////////////////////////
int main(
 int aArgc
,char** aArgv
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  int verboseLevel = 0;

  Lib::Debug::checkByClass(true);

 {//  Put everything in a block so that temporary objects
  // on the stack are deleted before the final memory decount is done. 

  Lib::Arguments args(aArgc,aArgv);
  bool help = args.isAnArgument("-h");
  if(help) {
    std::cout << "BatchLab_exa_analysis :" << std::endl;
    std::cout << "Options : " << std::endl;
    std::cout << "  OS> BatchLab_exa_analysis -h" << std::endl;
    std::cout << "   to have help." << std::endl;
    std::cout << "  OS> BatchLab_exa_analysis -verbose" << std::endl;
    std::cout << "   to switch to verbose mode." << std::endl;
    std::cout << "  OS> BatchLab_exa_analysis"
              << " -store=[memory,aida,zip,root,hbook]" 
              << std::endl;
    std::cout << "   to store :"
              << "     in memory"
              << "     in an aida xml file"
              << "     in a file at the ROOT format with BatchLab objects."
              << std::endl;
    std::cout << "  OS> BatchLab_exa_analysis -store=root -export=root" 
              << std::endl;
    std::cout << "   to store :"
              << "     in a file at the ROOT format with ROOT objects."
              << std::endl;
    std::cout << "  OS> BatchLab_exa_analysis -plotting" << std::endl;
    std::cout << "   to run without the plotting (batch mode)." << std::endl;
    std::cout << "  OS> BatchLab_exa_analysis -plotter=[BatchLab,Anaphe]" 
              << std::endl;
    std::cout << "   to use a plotter of another implementation." << std::endl;
    return 0;
  }

  std::string sverbose;
  if(args.find("-verbose",sverbose)) {
    if(sverbose=="") verboseLevel = 1;
  }

  AIDA::IAnalysisFactory* aida = AIDA_createAnalysisFactory();
  if(!aida) {
    std::cout << "BatchLab_exa_analysis :"
              << " Can't get an AnalysisFactory." << std::endl;
    return 1;
  }

  if(verboseLevel) {
    std::cout << "BatchLab_exa_analysis :"
              << " Have an AnalysisFactory." << std::endl;
  }

  AIDA::ITreeFactory* treeFactory = aida->createTreeFactory();
  if(!treeFactory) {
    std::cout << "BatchLab_exa_analysis :"
              << " Can't get a TreeFactory." << std::endl;
    return 1;
  }

  if(verboseLevel) {
    std::cout << "BatchLab_exa_analysis :"
              << " Have a TreeFactory." << std::endl;
  }

  // Create some random engines :
  Lib::RandomGauss randomGauss;
  Lib::RandomFlat randomFlat;
  Lib::RandomBreitWigner randomBreitWigner;

  bool first_tree = true;
  //bool first_tree = false;
  bool second_tree = true;

  // Decide storage type and options from arguments :
  std::string store;
  if(!args.find("-store",store)) store = "memory";
  Lib::smanip::tolowercase(store);
  bool compress  = args.isAnArgument("-compress");
  std::string sexport;
  if(!args.find("-export",sexport)) sexport = "BatchLab";
  if(verboseLevel) {
    std::cout << "BatchLab_exa_analysis :"
              << " Choosen store type is \"" << store << "\"." << std::endl;
  }

  //////////////////////////////////////////////////////////
  /// First tree :
  //////////////////////////////////////////////////////////
  if(first_tree) 
 {AIDA::ITree* tree = 0;
  if(store=="memory") {
    tree = treeFactory->create();
  } else {
    if(store=="root") {
      std::string opts = "compress=";
      opts += compress ? "yes" : "no";
      opts += ";export="+sexport;
      tree = treeFactory->create
               ("BatchLab_exa_analysis.root","root",false,true,opts);
    } else if(store=="xml") {
      std::string opts = "compress=";
      opts += compress ? "yes" : "no";
      tree = treeFactory->create
               ("BatchLab_exa_analysis.aida","xml",false,true,opts);
    } else if(store=="zip") {
      std::string opts;
      tree = treeFactory->create
               ("BatchLab_exa_analysis.zip","zip",false,true,opts);
    } else if(store=="fs") {
      std::string opts;
      //opts += "streaming=ascii";
      //opts += ";header=false";
      //opts += ";tab=true";
      tree = treeFactory->create
               ("BatchLab_exa_analysis.fs","fs",false,true,opts);
    } else if(store=="hdf5") {
      std::string opts;
      tree = treeFactory->create
               ("BatchLab_exa_analysis.hdf5","hdf5",false,true,opts);
    } else if(store=="ascii") {
      std::string opts;
      tree = treeFactory->create
               ("BatchLab_exa_analysis.ascii","ascii",false,true,opts);
    } else if((store=="hbook")||((store=="zebra"))) {
      tree = treeFactory->create
               ("BatchLab_exa_analysis.hbook","hbook",false,true);
    } else {
      std::cout << "BatchLab_exa_analysis :"
                << " store type " << store << " unkown." << std::endl;
      return 1;
    }
  }
  if(!tree) {
    std::cout << "BatchLab_exa_analysis :"
              << " Can't create the BatchLab_exa_analysis tree." << std::endl;
    return 1;
  }
  if(verboseLevel) {
    std::cout << "BatchLab_exa_analysis :"
              << " \"" << tree->storeName() << "\" created." << std::endl;
  }

  if(!tree->mkdir("histograms")) {
    std::cout << "BatchLab_exa_analysis :"
              << " \"histograms\" directory not created." << std::endl;
    return 1;
  }
  if(verboseLevel) {
    std::cout << "BatchLab_exa_analysis :"
              << " \"histograms\" directory created." << std::endl;
  }

  if(!tree->cd("histograms")) {
    std::cout << "BatchLab_exa_analysis :"
              << " can't cd to \"histograms\"." << std::endl;
    return 1;
  }
  if(verboseLevel) {
    std::cout << "BatchLab_exa_analysis :"
              << " cd to \"histograms\" done." << std::endl;
  }

  // Get an histogram factory :
  AIDA::IHistogramFactory* histoFactory = 
    aida->createHistogramFactory(*tree);
  if(!histoFactory) {
    std::cout << "BatchLab_exa_analysis :"
              << " Can't get a HistogramFactory." << std::endl;
    return 1;
  }

  if(verboseLevel) {
    std::cout << "BatchLab_exa_analysis :"
              << " Have a HistogramFactory." << std::endl;
  }

  ////////////////////////////
  // Create some histos :
  ////////////////////////////
  AIDA::IHistogram1D* gauss = 
    histoFactory->createHistogram1D("gauss","Random gauss",100,-5,5);
  if(!gauss) {
    std::cout << "BatchLab_exa_analysis :"
              << " Can't create an histogram." << std::endl;
    return 1;
  }

  if(verboseLevel) {
    std::cout << "BatchLab_exa_analysis :"
              << " \"gauss\" histogram created." << std::endl;
  }

 {for(int count=0;count<5000;count++) gauss->fill(randomGauss.shoot());}

  std::cout << "BatchLab_anslysis :"
            << "gauss :"
            << " entries " << gauss->entries() 
            << " mean " << gauss->mean() << " rms " << gauss->rms() 
            << std::endl;

  if(verboseLevel) {
    std::cout << "BatchLab_exa_analysis :"
              << " \"gauss\" histogram filled." << std::endl;
  }

  //FIXME
  //tree->commit();
  //delete aida;
  //exit(0);

  ////////////////////////////
  ////////////////////////////
  AIDA::IHistogram1D* flat = 
    histoFactory->createHistogram1D("10","Random flat",100,0,1);
  if(!flat) {
    std::cout << "BatchLab_exa_analysis :"
              << " Can't create the flat histogram." << std::endl;
    return 1;
  }

  if(verboseLevel) {
    std::cout << "BatchLab_exa_analysis :"
              << " \"flat\" histogram created." << std::endl;
  }

  {for(int count=0;count<5000;count++) flat->fill(randomFlat.shoot());}

  if(verboseLevel) {
    std::cout << "BatchLab_exa_analysis :"
              << " \"flat\" histogram filled." << std::endl;
  }

  ////////////////////////////
  ////////////////////////////
  AIDA::IHistogram2D* gauss_BW = 
    histoFactory->createHistogram2D("gauss_BW",
                                    "BW/Gauss",
                                    100,-5,5,
                                    100,-5,5);
  if(!gauss_BW) {
    std::cout << "BatchLab_exa_analysis :"
              << " Can't create the gauss_BW histogram." << std::endl;
    return 1;
  }

  if(verboseLevel) {
    std::cout << "BatchLab_exa_analysis :"
              << " \"gauss_BW\" histogram created." << std::endl;
  }

 {for(int count=0;count<1000;count++) {
    gauss_BW->fill(randomGauss.shoot(),randomBreitWigner.shoot());
  }}

  std::cout << "BatchLab_anslysis :"
            << "gauss_BW :"
            << " entries " << gauss_BW->entries() 
            << " meanX " << gauss_BW->meanX() << " rmsX " << gauss_BW->rmsX() 
            << " meanY " << gauss_BW->meanY() << " rmsY " << gauss_BW->rmsY() 
            << std::endl;

  if(verboseLevel) {
    std::cout << "BatchLab_exa_analysis :"
              << " \"gauss_BW\" histogram filled." << std::endl;
  }

  ////////////////////////////
  ////////////////////////////
  AIDA::IHistogram1D* BW = 
    histoFactory->createHistogram1D("BW","Random BW",100,-5,5);
  if(!BW) {
    std::cout << "BatchLab_exa_analysis :"
              << " Can't create the BW histogram." << std::endl;
    return 1;
  }

  if(verboseLevel) {
    std::cout << "BatchLab_exa_analysis :"
              << " \"BW\" histogram created." << std::endl;
  }

 {for(int count=0;count<5000;count++) {
    BW->fill(randomBreitWigner.shoot());
  }}

  if(verboseLevel) {
    std::cout << "BatchLab_exa_analysis :"
              << " \"BW\" histogram filled." << std::endl;
  }

  ////////////////////////////
  ////////////////////////////
  AIDA::IProfile1D* profile = 
    histoFactory->createProfile1D("profile",
                                  "Profile",
                                  100,-5,5,-2,2);
  if(!profile) {
    std::cout << "BatchLab_exa_analysis :"
              << " Can't create the Profile histogram." << std::endl;
  } else {
    if(verboseLevel) {
      std::cout << "BatchLab_exa_analysis :"
                << " \"profile\" histogram created." << std::endl;
    }
   {for(int count=0;count<100000;count++) {
      profile->fill(randomGauss.shoot(),randomBreitWigner.shoot());
    }}
    std::cout << "BatchLab_anslysis :"
              << "profile :"
              << " entries " << gauss_BW->entries() 
              << " meanX " << gauss_BW->meanX() 
              << " rmsX " << gauss_BW->rmsX() 
              << std::endl;
    if(verboseLevel) {
      std::cout << "BatchLab_exa_analysis :"
                << " \"profile\" histogram filled." << std::endl;
    }
  }

  ////////////////////////////
  ////////////////////////////
  std::vector<double> edges;
  edges.push_back(-4);
  edges.push_back(-3);
  edges.push_back(-2.5);
  edges.push_back(-2);
  edges.push_back(-1.75);
  edges.push_back(-1.5);
  edges.push_back(-1.25);
  edges.push_back(-1);
  edges.push_back(-0.875);
  edges.push_back(-0.75);
  edges.push_back(-0.625);
  edges.push_back(-0.5);
  edges.push_back(-0.375);
  edges.push_back(-0.25);
  edges.push_back(-0.125);
  edges.push_back(0);
  edges.push_back(0.125);
  edges.push_back(0.25);
  edges.push_back(0.375);
  edges.push_back(0.5);
  edges.push_back(0.625);
  edges.push_back(0.75);
  edges.push_back(0.875);
  edges.push_back(1);
  edges.push_back(1.25);
  edges.push_back(1.5);
  edges.push_back(1.75);
  edges.push_back(2);
  edges.push_back(2.5);
  edges.push_back(3);
  edges.push_back(4);
  AIDA::IHistogram1D* gauss_vabins = 
    histoFactory->createHistogram1D("gauss_vabins",
                                    "Random gauss variable bins",
                                    edges);
  if(!gauss_vabins) {
    std::cout << "BatchLab_exa_analysis :"
              << " Can't create va bins histogram." << std::endl;
  } else {
    if(verboseLevel) {
      std::cout << "BatchLab_exa_analysis :"
                << " \"gauss_vabins\" histogram created." << std::endl;
    }

    Lib::RandomGauss randomGauss2(0,2);
   {for(int count=0;count<5000;count++) 
      gauss_vabins->fill(randomGauss2.shoot());}

    if(verboseLevel) {
      std::cout << "BatchLab_exa_analysis :"
                << " \"gauss_vabins\" histogram filled." << std::endl;
    }
  }

  delete histoFactory; // An histo factory is not "managed", we have to 
                       // delete it ourself.

  ////////////////////////////
  // Plotting :
  ////////////////////////////
  if(args.isAnArgument("-plotting")) {
    int argc = 1;
    static char* argv[] = {(char*)"-stdout"};

    std::string splotter;
    args.find("-plotter",splotter);

    if(verboseLevel) {
      std::cout << "BatchLab_exa_analysis :"
                << " choosen plotter \"" << splotter << "\"." << std::endl;
    }
    AIDA::IPlotterFactory* plotterFactory = 0;
    if(splotter=="BatchLab") {
      plotterFactory = 
        aida->createPlotterFactory(argc,argv,
          splotter,
          "/projects/BatchLab/v10r0/Linux-i686/libBatchLabCore.so");
    } else if(splotter=="Anaphe") {
      plotterFactory = 
        aida->createPlotterFactory(argc,argv,
          splotter,
          "/projects/Build/Anaphe/v1r0/Linux-i686/libAIDA_Plotter_Grace.so");
    } else {
      // Plotter of the implementation attached 
      // with AIDA_createAnalysisFactory :
      plotterFactory = aida->createPlotterFactory(argc,argv);
    }
    if(!plotterFactory) {
      std::cout << "BatchLab_exa_analysis :"
                << " Can't get a PlotterFactory." << std::endl;
      return 1;
    } else {
      if(verboseLevel) {
        std::cout << "BatchLab_exa_analysis :"
                  << " Have a PlotterFactory." << std::endl;
      }
    
      AIDA::IPlotter* plotter = plotterFactory->create();
      if(!plotter) {
        std::cout << "BatchLab_exa_analysis :"
                  << " Can't get a plotter." << std::endl;
      } else {
        if(verboseLevel) {
           std::cout << "BatchLab_exa_analysis :"
                    << " Have a Plotter." << std::endl;
        }
    
        plotter->show();
        plotter->clearRegions();
        AIDA::IPlotterRegion& region = plotter->currentRegion();
        region.plot(*BW);
        std::cout << "BatchLab_exa_analysis :"
                  << " Click the escape (or exit, quit) button to continue..." 
                  << std::endl;
        // The user have to click in some "escape GUI" button
        // to return from "interact".
        plotter->interact();
        plotter->hide();
       {std::cout << "Type c and return to continue" << std::endl;
        char c;
        std::cin >> c;}   

        // Demonstrate how to update the plotter :
        region.clear();
        region.plot(*gauss_BW);
    
        if(args.isAnArgument("-xyModeling=color")) {
          region.setParameter("xyModeling","COLOR");
        }
        //plotter->setParameter("infosRegionVisible","TRUE");
        // [0 , 10[ = white ; [10, 50[ = yellow ; [50,oo[ red :
        region.setParameter("colorMapping","yellow 10 orange 40 red ");
        for(int update = 0;update<3;update++) {
          // New data :
         {for(int count = 0;count<5000;count++) {
            gauss_BW->fill(randomGauss.shoot(),randomBreitWigner.shoot());
          }}
          // Update the plotter :
          plotter->refresh();
          std::cout << "BatchLab_exa_analysis :"
                    << " Click the escape (or exit, quit) button to continue ("
                    << update
                    << ")..."
                    << std::endl;
          plotter->show();
          plotter->interact();
          plotter->hide();
         {std::cout << "Type c and return to continue" << std::endl;
          char c;
          std::cin >> c;}
        }
        if(verboseLevel) {
          std::cout << "BatchLab_exa_analysis :"
                    << " End plotting." << std::endl;
        }
    
        delete plotter;
      }
      delete plotterFactory;
    }
  }

  if(!tree->cd("..")) {
    std::cout << "BatchLab_exa_analysis :"
              << " can't cd to \"..\"." << std::endl;
    return 1;
  }

  if(!tree->mkdir("tuples")) {
    std::cout << "BatchLab_exa_analysis :"
              << " can't make directory \"tuples\"." << std::endl;
    return 1;
  }

  if(!tree->cd("tuples")) {
    std::cout << "BatchLab_exa_analysis :"
              << " can't cd to \"tuples\"." << std::endl;
    return 1;
  }

  std::cout << "BatchLab_exa_analysis :"
            << " pwd \"" << tree->pwd() << "\"." << std::endl;

  // Get a tuple factory :
  AIDA::ITupleFactory* tupleFactory = 
    aida->createTupleFactory(*tree);
  if(!tupleFactory) {
    std::cout << "BatchLab_exa_analysis :"
              << " Can't get a TupleFactory." << std::endl;
    return 1;
  }

  if(verboseLevel) {
    std::cout << "BatchLab_exa_analysis :"
              << " Have a TupleFactory." << std::endl;
  }

  // Create some tuples :
  AIDA::ITuple* tuple1 = tupleFactory->create("tuple1","tuple1",
                                              "double v1, double v2");
  if(!tuple1) {
    std::cout << "BatchLab_exa_analysis :"
              << " Can't create the \"tuple1\" tuple." << std::endl;
    return 1;
  }

  if(verboseLevel) {
    std::cout << "BatchLab_exa_analysis :"
              << " \"tuple1\" created." << std::endl;
  }

  // Fill so that an histo will give a step function :
  // The tuple will have 2 columns (filled with same data)
  // and :
  //     0 + 1 + 2 + .... + (n-1) = (n-1) * n/2
  // rows. Then below : n = 10, then 45 rows.
 {for(int count=0;count<10;count++) {
    for(int i=0;i<count;i++) {
      tuple1->fill(0,(double)count);
      tuple1->fill(1,(double)count);
      tuple1->addRow();
    }
  }}

  // Test the ';' booking way :
  AIDA::ITuple* tuple2 = 
    tupleFactory->create("tuple2","tuple2",
               "double rand_gauss; double rand_BW, step;");
  if(!tuple2) {
    std::cout << "BatchLab_exa_analysis :"
              << " Can't create the \"tuple2\" tuple." << std::endl;
    return 1;
  }


  if(verboseLevel) {
    std::cout << "BatchLab_exa_analysis :"
              << " \"tuple2\" created." << std::endl;
  }

 {for(int count=0;count<5000;count++) {
    tuple2->fill(0,randomGauss.shoot());
    tuple2->fill(1,randomBreitWigner.shoot());
    tuple2->fill(2,(double)count);
    tuple2->addRow();
  }}

  tree->cd("..");

  std::cout << "BatchLab_exa_analysis :"
            << " Produce \"" << tree->storeName() << "\"..." << std::endl;

  if(!tree->commit()) {
    std::cout << "BatchLab_exa_analysis :"
              << " can't commit." << std::endl;
    return 1;
  }

  tree->ls(".",true);

  std::cout << "BatchLab_exa_analysis :"
            << " pwd \"" << tree->pwd() << "\"." << std::endl;

  delete tupleFactory;
  if(store!="memory") delete tree;
  }

  //////////////////////////////////////////////////////////
  /// Second tree :
  //////////////////////////////////////////////////////////
  if(second_tree)
 {AIDA::ITree* tree = 0;
  if(store=="memory") {
    tree = treeFactory->create();
  } else {
    if(store=="root") {
      std::string opts = "compress=";
      opts += compress ? "yes" : "no";
      opts += ";export="+sexport;
      tree = treeFactory->create
               ("BatchLab_exa_analysis1.root","root",false,true,opts);
    } else if(store=="xml") {
      std::string opts = "compress=";
      opts += compress ? "yes" : "no";
      tree = treeFactory->create
               ("BatchLab_exa_analysis1.aida","xml",false,true,opts);
    } else if(store=="zip") {
      std::string opts;
      tree = treeFactory->create
               ("BatchLab_exa_analysis1.zip","zip",false,true,opts);
    } else if(store=="fs") {
      std::string opts;
      //opts += "streaming=ascii";
      tree = treeFactory->create
               ("BatchLab_exa_analysis1.fs","fs",false,true,opts);
    } else if(store=="hdf5") {
      std::string opts;
      tree = treeFactory->create
               ("BatchLab_exa_analysis1.hdf5","hdf5",false,true,opts);
    } else if(store=="ascii") {
      std::string opts;
      tree = treeFactory->create
               ("BatchLab_exa_analysis1.ascii","ascii",false,true,opts);
    } else if(store=="hbook") {
      tree = treeFactory->create
               ("BatchLab_exa_analysis1.hbook","hbook",false,true);
    }
  }
  if(!tree) {
    std::cout << "BatchLab_exa_analysis :"
              << " Can't create the BatchLab_exa_analysis1 tree." 
              << std::endl;
    return 1;
  }

  if(verboseLevel) {
    std::cout << "BatchLab_exa_analysis :"
              << " \"" << tree->storeName() << "\" created." << std::endl;
  }

  tree->mkdir("tuples");

  if(verboseLevel) {
    std::cout << "BatchLab_exa_analysis :"
              << " \"tuples\" directory created." << std::endl;
  }

  tree->cd("tuples");

  if(verboseLevel) {
    std::cout << "BatchLab_exa_analysis :"
              << " cd to \"tuples\" done." << std::endl;
  }


  AIDA::ITupleFactory* tupleFactory = 
    aida->createTupleFactory(*tree);
  if(!tupleFactory) {
    std::cout << "BatchLab_exa_analysis :"
              << " Can't create a TupleFactory." << std::endl;
    return 1;
  }

  if(verboseLevel) {
    std::cout << "BatchLab_exa_analysis :"
              << " Have a TupleFactory." << std::endl;
  }

  AIDA::ITuple* tuple = 
    tupleFactory->create("tuple","tuple","double rand_gauss, double rand_BW");
  if(!tuple) {
    std::cout << "BatchLab_exa_analysis :"
              << " Can't create the \"tuple\" tuple." << std::endl;
    return 1;
  }

  if(verboseLevel) {
    std::cout << "BatchLab_exa_analysis :"
              << " \"tuple\" created." << std::endl;
  }

 {for(int count=0;count<100000;count++) {
    tuple->fill(0,randomGauss.shoot());
    tuple->fill(1,randomBreitWigner.shoot());
    tuple->addRow();
  }}

  tree->cd("..");

  std::cout << "BatchLab_exa_analysis :"
            << " Produce \"" << tree->storeName() << "\"..." << std::endl;
  tree->commit();
  tree->ls(".",true);
  delete tupleFactory;
  delete tree;}

  delete treeFactory;

  if(verboseLevel) {
    std::cout << "BatchLab_exa_analysis :"
              << " Delete the AnalysisFactory..." << std::endl;
  }

  delete aida; // Will delete the trees.
  }

  // A simple check to see if object creation/destruction balance is correct :
  Lib::Printer printer;
  Lib::Debug::balance(printer);

  if(verboseLevel) {
    std::cout << "BatchLab_exa_analysis :"
              << " Exiting..." << std::endl;
  }

  return 0;
}

